// Targeted by JavaCPP version 0.10

package org.bytedeco.javacpp;

import java.nio.*;
import org.bytedeco.javacpp.*;
import org.bytedeco.javacpp.annotation.*;

import static org.bytedeco.javacpp.avutil.*;
import static org.bytedeco.javacpp.swresample.*;
import static org.bytedeco.javacpp.avcodec.*;
import static org.bytedeco.javacpp.avformat.*;
import static org.bytedeco.javacpp.postproc.*;
import static org.bytedeco.javacpp.swscale.*;
import static org.bytedeco.javacpp.avfilter.*;

public class avdevice extends org.bytedeco.javacpp.presets.avdevice {
    static { Loader.load(); }

// Parsed from <libavdevice/avdevice.h>

/*
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

// #ifndef AVDEVICE_AVDEVICE_H
// #define AVDEVICE_AVDEVICE_H

// #include "version.h"

/**
 * @file
 * @ingroup lavd
 * Main libavdevice API header
 */

/**
 * @defgroup lavd Special devices muxing/demuxing library
 * @{
 * Libavdevice is a complementary library to @ref libavf "libavformat". It
 * provides various "special" platform-specific muxers and demuxers, e.g. for
 * grabbing devices, audio capture and playback etc. As a consequence, the
 * (de)muxers in libavdevice are of the AVFMT_NOFILE type (they use their own
 * I/O functions). The filename passed to avformat_open_input() often does not
 * refer to an actually existing file, but has some special device-specific
 * meaning - e.g. for x11grab it is the display name.
 *
 * To use libavdevice, simply call avdevice_register_all() to register all
 * compiled muxers and demuxers. They all use standard libavformat API.
 * @}
 */

// #include "libavutil/log.h"
// #include "libavutil/opt.h"
// #include "libavutil/dict.h"
// #include "libavformat/avformat.h"

/**
 * Return the LIBAVDEVICE_VERSION_INT constant.
 */
public static native @Cast("unsigned") int avdevice_version();

/**
 * Return the libavdevice build-time configuration.
 */
public static native @Cast("const char*") BytePointer avdevice_configuration();

/**
 * Return the libavdevice license.
 */
public static native @Cast("const char*") BytePointer avdevice_license();

/**
 * Initialize libavdevice and register all the input and output devices.
 * @warning This function is not thread safe.
 */
public static native void avdevice_register_all();

/**
 * Audio input devices iterator.
 *
 * If d is NULL, returns the first registered input audio/video device,
 * if d is non-NULL, returns the next registered input audio/video device after d
 * or NULL if d is the last one.
 */


/**
 * Video input devices iterator.
 *
 * If d is NULL, returns the first registered input audio/video device,
 * if d is non-NULL, returns the next registered input audio/video device after d
 * or NULL if d is the last one.
 */


/**
 * Audio output devices iterator.
 *
 * If d is NULL, returns the first registered output audio/video device,
 * if d is non-NULL, returns the next registered output audio/video device after d
 * or NULL if d is the last one.
 */


/**
 * Video output devices iterator.
 *
 * If d is NULL, returns the first registered output audio/video device,
 * if d is non-NULL, returns the next registered output audio/video device after d
 * or NULL if d is the last one.
 */


public static class AVDeviceRect extends Pointer {
    static { Loader.load(); }
    public AVDeviceRect() { allocate(); }
    public AVDeviceRect(int size) { allocateArray(size); }
    public AVDeviceRect(Pointer p) { super(p); }
    private native void allocate();
    private native void allocateArray(int size);
    @Override public AVDeviceRect position(int position) {
        return (AVDeviceRect)super.position(position);
    }

    /** x coordinate of top left corner */
    public native int x(); public native AVDeviceRect x(int x);
    /** y coordinate of top left corner */
    public native int y(); public native AVDeviceRect y(int y);
    /** width */
    public native int width(); public native AVDeviceRect width(int width);
    /** height */
    public native int height(); public native AVDeviceRect height(int height);
}

/**
 * Message types used by avdevice_app_to_dev_control_message().
 */
/** enum AVAppToDevMessageType */

public static native @MemberGetter int AV_APP_TO_DEV_NONE();
public static final int
    /**
     * Dummy message.
     */
    AV_APP_TO_DEV_NONE = AV_APP_TO_DEV_NONE();
public static native @MemberGetter int AV_APP_TO_DEV_WINDOW_SIZE();
public static final int

    /**
     * Window size change message.
     *
     * Message is sent to the device every time the application changes the size
     * of the window device renders to.
     * Message should also be sent right after window is created.
     *
     * data: AVDeviceRect: new window size.
     */
    AV_APP_TO_DEV_WINDOW_SIZE = AV_APP_TO_DEV_WINDOW_SIZE();
public static native @MemberGetter int AV_APP_TO_DEV_WINDOW_REPAINT();
public static final int

    /**
     * Repaint request message.
     *
     * Message is sent to the device when window has to be repainted.
     *
     * data: AVDeviceRect: area required to be repainted.
     *       NULL: whole area is required to be repainted.
     */
    AV_APP_TO_DEV_WINDOW_REPAINT = AV_APP_TO_DEV_WINDOW_REPAINT();
public static native @MemberGetter int AV_APP_TO_DEV_PAUSE();
public static final int

    /**
     * Request pause/play.
     *
     * Application requests pause/unpause playback.
     * Mostly usable with devices that have internal buffer.
     * By default devices are not paused.
     *
     * data: NULL
     */
    AV_APP_TO_DEV_PAUSE        = AV_APP_TO_DEV_PAUSE();
public static native @MemberGetter int AV_APP_TO_DEV_PLAY();
public static final int
    AV_APP_TO_DEV_PLAY         = AV_APP_TO_DEV_PLAY();
public static native @MemberGetter int AV_APP_TO_DEV_TOGGLE_PAUSE();
public static final int
    AV_APP_TO_DEV_TOGGLE_PAUSE = AV_APP_TO_DEV_TOGGLE_PAUSE();
public static native @MemberGetter int AV_APP_TO_DEV_SET_VOLUME();
public static final int

    /**
     * Volume control message.
     *
     * Set volume level. It may be device-dependent if volume
     * is changed per stream or system wide. Per stream volume
     * change is expected when possible.
     *
     * data: double: new volume with range of 0.0 - 1.0.
     */
    AV_APP_TO_DEV_SET_VOLUME = AV_APP_TO_DEV_SET_VOLUME();
public static native @MemberGetter int AV_APP_TO_DEV_MUTE();
public static final int

    /**
     * Mute control messages.
     *
     * Change mute state. It may be device-dependent if mute status
     * is changed per stream or system wide. Per stream mute status
     * change is expected when possible.
     *
     * data: NULL.
     */
    AV_APP_TO_DEV_MUTE        = AV_APP_TO_DEV_MUTE();
public static native @MemberGetter int AV_APP_TO_DEV_UNMUTE();
public static final int
    AV_APP_TO_DEV_UNMUTE      = AV_APP_TO_DEV_UNMUTE();
public static native @MemberGetter int AV_APP_TO_DEV_TOGGLE_MUTE();
public static final int
    AV_APP_TO_DEV_TOGGLE_MUTE = AV_APP_TO_DEV_TOGGLE_MUTE();
public static native @MemberGetter int AV_APP_TO_DEV_GET_VOLUME();
public static final int

    /**
     * Get volume/mute messages.
     *
     * Force the device to send AV_DEV_TO_APP_VOLUME_LEVEL_CHANGED or
     * AV_DEV_TO_APP_MUTE_STATE_CHANGED command respectively.
     *
     * data: NULL.
     */
    AV_APP_TO_DEV_GET_VOLUME = AV_APP_TO_DEV_GET_VOLUME();
public static native @MemberGetter int AV_APP_TO_DEV_GET_MUTE();
public static final int
    AV_APP_TO_DEV_GET_MUTE   = AV_APP_TO_DEV_GET_MUTE();

/**
 * Message types used by avdevice_dev_to_app_control_message().
 */
/** enum AVDevToAppMessageType */

public static native @MemberGetter int AV_DEV_TO_APP_NONE();
public static final int
    /**
     * Dummy message.
     */
    AV_DEV_TO_APP_NONE = AV_DEV_TO_APP_NONE();
public static native @MemberGetter int AV_DEV_TO_APP_CREATE_WINDOW_BUFFER();
public static final int

    /**
     * Create window buffer message.
     *
     * Device requests to create a window buffer. Exact meaning is device-
     * and application-dependent. Message is sent before rendering first
     * frame and all one-shot initializations should be done here.
     * Application is allowed to ignore preferred window buffer size.
     *
     * @note: Application is obligated to inform about window buffer size
     *        with AV_APP_TO_DEV_WINDOW_SIZE message.
     *
     * data: AVDeviceRect: preferred size of the window buffer.
     *       NULL: no preferred size of the window buffer.
     */
    AV_DEV_TO_APP_CREATE_WINDOW_BUFFER = AV_DEV_TO_APP_CREATE_WINDOW_BUFFER();
public static native @MemberGetter int AV_DEV_TO_APP_PREPARE_WINDOW_BUFFER();
public static final int

    /**
     * Prepare window buffer message.
     *
     * Device requests to prepare a window buffer for rendering.
     * Exact meaning is device- and application-dependent.
     * Message is sent before rendering of each frame.
     *
     * data: NULL.
     */
    AV_DEV_TO_APP_PREPARE_WINDOW_BUFFER = AV_DEV_TO_APP_PREPARE_WINDOW_BUFFER();
public static native @MemberGetter int AV_DEV_TO_APP_DISPLAY_WINDOW_BUFFER();
public static final int

    /**
     * Display window buffer message.
     *
     * Device requests to display a window buffer.
     * Message is sent when new frame is ready to be displayed.
     * Usually buffers need to be swapped in handler of this message.
     *
     * data: NULL.
     */
    AV_DEV_TO_APP_DISPLAY_WINDOW_BUFFER = AV_DEV_TO_APP_DISPLAY_WINDOW_BUFFER();
public static native @MemberGetter int AV_DEV_TO_APP_DESTROY_WINDOW_BUFFER();
public static final int

    /**
     * Destroy window buffer message.
     *
     * Device requests to destroy a window buffer.
     * Message is sent when device is about to be destroyed and window
     * buffer is not required anymore.
     *
     * data: NULL.
     */
    AV_DEV_TO_APP_DESTROY_WINDOW_BUFFER = AV_DEV_TO_APP_DESTROY_WINDOW_BUFFER();
public static native @MemberGetter int AV_DEV_TO_APP_BUFFER_OVERFLOW();
public static final int

    /**
     * Buffer fullness status messages.
     *
     * Device signals buffer overflow/underflow.
     *
     * data: NULL.
     */
    AV_DEV_TO_APP_BUFFER_OVERFLOW = AV_DEV_TO_APP_BUFFER_OVERFLOW();
public static native @MemberGetter int AV_DEV_TO_APP_BUFFER_UNDERFLOW();
public static final int
    AV_DEV_TO_APP_BUFFER_UNDERFLOW = AV_DEV_TO_APP_BUFFER_UNDERFLOW();
public static native @MemberGetter int AV_DEV_TO_APP_BUFFER_READABLE();
public static final int

    /**
     * Buffer readable/writable.
     *
     * Device informs that buffer is readable/writable.
     * When possible, device informs how many bytes can be read/write.
     *
     * @warning Device may not inform when number of bytes than can be read/write changes.
     *
     * data: int64_t: amount of bytes available to read/write.
     *       NULL: amount of bytes available to read/write is not known.
     */
    AV_DEV_TO_APP_BUFFER_READABLE = AV_DEV_TO_APP_BUFFER_READABLE();
public static native @MemberGetter int AV_DEV_TO_APP_BUFFER_WRITABLE();
public static final int
    AV_DEV_TO_APP_BUFFER_WRITABLE = AV_DEV_TO_APP_BUFFER_WRITABLE();
public static native @MemberGetter int AV_DEV_TO_APP_MUTE_STATE_CHANGED();
public static final int

    /**
     * Mute state change message.
     *
     * Device informs that mute state has changed.
     *
     * data: int: 0 for not muted state, non-zero for muted state.
     */
    AV_DEV_TO_APP_MUTE_STATE_CHANGED = AV_DEV_TO_APP_MUTE_STATE_CHANGED();
public static native @MemberGetter int AV_DEV_TO_APP_VOLUME_LEVEL_CHANGED();
public static final int

    /**
     * Volume level change message.
     *
     * Device informs that volume level has changed.
     *
     * data: double: new volume with range of 0.0 - 1.0.
     */
    AV_DEV_TO_APP_VOLUME_LEVEL_CHANGED = AV_DEV_TO_APP_VOLUME_LEVEL_CHANGED();

/**
 * Send control message from application to device.
 *
 * @param s         device context.
 * @param type      message type.
 * @param data      message data. Exact type depends on message type.
 * @param data_size size of message data.
 * @return >= 0 on success, negative on error.
 *         AVERROR(ENOSYS) when device doesn't implement handler of the message.
 */
public static native int avdevice_app_to_dev_control_message(AVFormatContext s,
                                        @Cast("AVAppToDevMessageType") int type,
                                        Pointer data, @Cast("size_t") long data_size);

/**
 * Send control message from device to application.
 *
 * @param s         device context.
 * @param type      message type.
 * @param data      message data. Can be NULL.
 * @param data_size size of message data.
 * @return >= 0 on success, negative on error.
 *         AVERROR(ENOSYS) when application doesn't implement handler of the message.
 */
public static native int avdevice_dev_to_app_control_message(AVFormatContext s,
                                        @Cast("AVDevToAppMessageType") int type,
                                        Pointer data, @Cast("size_t") long data_size);

/**
 * Following API allows user to probe device capabilities (supported codecs,
 * pixel formats, sample formats, resolutions, channel counts, etc).
 * It is build on top op AVOption API.
 * Queried capabilities allows to set up converters of video or audio
 * parameters that fit to the device.
 *
 * List of capabilities that can be queried:
 *  - Capabilities valid for both audio and video devices:
 *    - codec:          supported audio/video codecs.
 *                      type: AV_OPT_TYPE_INT (AVCodecID value)
 *  - Capabilities valid for audio devices:
 *    - sample_format:  supported sample formats.
 *                      type: AV_OPT_TYPE_INT (AVSampleFormat value)
 *    - sample_rate:    supported sample rates.
 *                      type: AV_OPT_TYPE_INT
 *    - channels:       supported number of channels.
 *                      type: AV_OPT_TYPE_INT
 *    - channel_layout: supported channel layouts.
 *                      type: AV_OPT_TYPE_INT64
 *  - Capabilities valid for video devices:
 *    - pixel_format:   supported pixel formats.
 *                      type: AV_OPT_TYPE_INT (AVPixelFormat value)
 *    - window_size:    supported window sizes (describes size of the window size presented to the user).
 *                      type: AV_OPT_TYPE_IMAGE_SIZE
 *    - frame_size:     supported frame sizes (describes size of provided video frames).
 *                      type: AV_OPT_TYPE_IMAGE_SIZE
 *    - fps:            supported fps values
 *                      type: AV_OPT_TYPE_RATIONAL
 *
 * Value of the capability may be set by user using av_opt_set() function
 * and AVDeviceCapabilitiesQuery object. Following queries will
 * limit results to the values matching already set capabilities.
 * For example, setting a codec may impact number of formats or fps values
 * returned during next query. Setting invalid value may limit results to zero.
 *
 * Example of the usage basing on opengl output device:
 *
 * @code
 *  AVFormatContext *oc = NULL;
 *  AVDeviceCapabilitiesQuery *caps = NULL;
 *  AVOptionRanges *ranges;
 *  int ret;
 *
 *  if ((ret = avformat_alloc_output_context2(&oc, NULL, "opengl", NULL)) < 0)
 *      goto fail;
 *  if (avdevice_capabilities_create(&caps, oc, NULL) < 0)
 *      goto fail;
 *
 *  //query codecs
 *  if (av_opt_query_ranges(&ranges, caps, "codec", AV_OPT_MULTI_COMPONENT_RANGE)) < 0)
 *      goto fail;
 *  //pick codec here and set it
 *  av_opt_set(caps, "codec", AV_CODEC_ID_RAWVIDEO, 0);
 *
 *  //query format
 *  if (av_opt_query_ranges(&ranges, caps, "pixel_format", AV_OPT_MULTI_COMPONENT_RANGE)) < 0)
 *      goto fail;
 *  //pick format here and set it
 *  av_opt_set(caps, "pixel_format", AV_PIX_FMT_YUV420P, 0);
 *
 *  //query and set more capabilities
 *
 * fail:
 *  //clean up code
 *  avdevice_capabilities_free(&query, oc);
 *  avformat_free_context(oc);
 * @endcode
 */

/**
 * Structure describes device capabilities.
 *
 * It is used by devices in conjunction with av_device_capabilities AVOption table
 * to implement capabilities probing API based on AVOption API. Should not be used directly.
 */
public static class AVDeviceCapabilitiesQuery extends Pointer {
    static { Loader.load(); }
    public AVDeviceCapabilitiesQuery() { allocate(); }
    public AVDeviceCapabilitiesQuery(int size) { allocateArray(size); }
    public AVDeviceCapabilitiesQuery(Pointer p) { super(p); }
    private native void allocate();
    private native void allocateArray(int size);
    @Override public AVDeviceCapabilitiesQuery position(int position) {
        return (AVDeviceCapabilitiesQuery)super.position(position);
    }

    @MemberGetter public native @Const AVClass av_class();
    public native AVFormatContext device_context(); public native AVDeviceCapabilitiesQuery device_context(AVFormatContext device_context);
    public native @Cast("AVCodecID") int codec(); public native AVDeviceCapabilitiesQuery codec(int codec);
    public native @Cast("AVSampleFormat") int sample_format(); public native AVDeviceCapabilitiesQuery sample_format(int sample_format);
    public native @Cast("AVPixelFormat") int pixel_format(); public native AVDeviceCapabilitiesQuery pixel_format(int pixel_format);
    public native int sample_rate(); public native AVDeviceCapabilitiesQuery sample_rate(int sample_rate);
    public native int channels(); public native AVDeviceCapabilitiesQuery channels(int channels);
    public native long channel_layout(); public native AVDeviceCapabilitiesQuery channel_layout(long channel_layout);
    public native int window_width(); public native AVDeviceCapabilitiesQuery window_width(int window_width);
    public native int window_height(); public native AVDeviceCapabilitiesQuery window_height(int window_height);
    public native int frame_width(); public native AVDeviceCapabilitiesQuery frame_width(int frame_width);
    public native int frame_height(); public native AVDeviceCapabilitiesQuery frame_height(int frame_height);
    public native @ByRef AVRational fps(); public native AVDeviceCapabilitiesQuery fps(AVRational fps);
}

/**
 * AVOption table used by devices to implement device capabilities API. Should not be used by a user.
 */


/**
 * Initialize capabilities probing API based on AVOption API.
 *
 * avdevice_capabilities_free() must be called when query capabilities API is
 * not used anymore.
 *
 * @param[out] caps      Device capabilities data. Pointer to a NULL pointer must be passed.
 * @param s              Context of the device.
 * @param device_options An AVDictionary filled with device-private options.
 *                       On return this parameter will be destroyed and replaced with a dict
 *                       containing options that were not found. May be NULL.
 *                       The same options must be passed later to avformat_write_header() for output
 *                       devices or avformat_open_input() for input devices, or at any other place
 *                       that affects device-private options.
 *
 * @return >= 0 on success, negative otherwise.
 */
public static native int avdevice_capabilities_create(@Cast("AVDeviceCapabilitiesQuery**") PointerPointer caps, AVFormatContext s,
                                 @Cast("AVDictionary**") PointerPointer device_options);
public static native int avdevice_capabilities_create(@ByPtrPtr AVDeviceCapabilitiesQuery caps, AVFormatContext s,
                                 @ByPtrPtr AVDictionary device_options);

/**
 * Free resources created by avdevice_capabilities_create()
 *
 * @param caps Device capabilities data to be freed.
 * @param s    Context of the device.
 */
public static native void avdevice_capabilities_free(@Cast("AVDeviceCapabilitiesQuery**") PointerPointer caps, AVFormatContext s);
public static native void avdevice_capabilities_free(@ByPtrPtr AVDeviceCapabilitiesQuery caps, AVFormatContext s);

/**
 * Structure describes basic parameters of the device.
 */
public static class AVDeviceInfo extends Pointer {
    static { Loader.load(); }
    public AVDeviceInfo() { allocate(); }
    public AVDeviceInfo(int size) { allocateArray(size); }
    public AVDeviceInfo(Pointer p) { super(p); }
    private native void allocate();
    private native void allocateArray(int size);
    @Override public AVDeviceInfo position(int position) {
        return (AVDeviceInfo)super.position(position);
    }

    /** device name, format depends on device */
    public native @Cast("char*") BytePointer device_name(); public native AVDeviceInfo device_name(BytePointer device_name);
    /** human friendly name */
    public native @Cast("char*") BytePointer device_description(); public native AVDeviceInfo device_description(BytePointer device_description);
}

/**
 * List of devices.
 */
public static class AVDeviceInfoList extends Pointer {
    static { Loader.load(); }
    public AVDeviceInfoList() { allocate(); }
    public AVDeviceInfoList(int size) { allocateArray(size); }
    public AVDeviceInfoList(Pointer p) { super(p); }
    private native void allocate();
    private native void allocateArray(int size);
    @Override public AVDeviceInfoList position(int position) {
        return (AVDeviceInfoList)super.position(position);
    }

    /** list of autodetected devices */
    public native AVDeviceInfo devices(int i); public native AVDeviceInfoList devices(int i, AVDeviceInfo devices);
    @MemberGetter public native @Cast("AVDeviceInfo**") PointerPointer devices();
    /** number of autodetected devices */
    public native int nb_devices(); public native AVDeviceInfoList nb_devices(int nb_devices);
    /** index of default device or -1 if no default */
    public native int default_device(); public native AVDeviceInfoList default_device(int default_device);
}

/**
 * List devices.
 *
 * Returns available device names and their parameters.
 *
 * @note: Some devices may accept system-dependent device names that cannot be
 *        autodetected. The list returned by this function cannot be assumed to
 *        be always completed.
 *
 * @param s                device context.
 * @param[out] device_list list of autodetected devices.
 * @return count of autodetected devices, negative on error.
 */
public static native int avdevice_list_devices(AVFormatContext s, @Cast("AVDeviceInfoList**") PointerPointer device_list);
public static native int avdevice_list_devices(AVFormatContext s, @ByPtrPtr AVDeviceInfoList device_list);

/**
 * Convenient function to free result of avdevice_list_devices().
 *
 * @param devices device list to be freed.
 */
public static native void avdevice_free_list_devices(@Cast("AVDeviceInfoList**") PointerPointer device_list);
public static native void avdevice_free_list_devices(@ByPtrPtr AVDeviceInfoList device_list);

// #endif /* AVDEVICE_AVDEVICE_H */


}
